unit OTFEFreeOTFE_frmDriverControl;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
  registry,
  OTFEFreeOTFE_DriverControl,
  WinSVC,  // Required for service related definitions and functions
  OTFEFreeOTFE_U;

type
  TfrmDriverControl = class(TForm)
    pbClose: TButton;
    gbInstallNew: TGroupBox;
    pbInstall: TButton;
    lblInstall: TLabel;
    gbModifyExisting: TGroupBox;
    lbDrivers: TListBox;
    lblStart: TLabel;
    pbStart: TButton;
    pbStop: TButton;
    pbUninstall: TButton;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    pbUpdate: TButton;
    cbStartup: TComboBox;
    OpenDialog: TOpenDialog;
    procedure lbDriversClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure pbStartClick(Sender: TObject);
    procedure pbStopClick(Sender: TObject);
    procedure pbUninstallClick(Sender: TObject);
    procedure pbInstallClick(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbStartupChange(Sender: TObject);
    procedure pbUpdateClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure lbDriversDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    DriverControlObj: TOTFEFreeOTFEDriverControl;

    BMPModeNormal: TBitmap;
    BMPModePortable: TBitmap;
    BMPStatusStarted: TBitmap;
    BMPStatusStopped: TBitmap;
    BMPStartAuto: TBitmap;
    BMPStartManual: TBitmap;

    procedure EnableDisableControls();

    procedure PopulateDriversList();

    procedure LoadBitmapsFromResources();
    procedure FreeBitmapsFromResources();

    function  GetItemWidth(Index: Integer): integer;
  public
    constructor Create(AOwner: TComponent); override;
  end;


implementation

{$R *.DFM}
{$R OTFEFreeOTFE_DriverImages.dcr}

uses
  SDUGeneral,
  SDUDialogs,
  Math;  // Required for max(...)


const
  TEXT_MANUAL_START = 'Manual';
  TEXT_AUTO_START   = 'At system startup';

  CRLF = #13 + #10;

  TEXT_MAY_NEED_ADMIN =
           'You may need administrator privileges in order to carry out'+CRLF+
           'this operation.';

  INTER_IMAGE_GAP = 2;

constructor TfrmDriverControl.Create(AOwner: TComponent);
begin
  inherited;

  // We create the driver control object *here*. We can't create it in
  // OnFormCreate(...) as any exception raised by TOTFEFreeOTFEDriverControl as
  // a result of it's creation (e.g. user doesn't have admin privs) would just
  // get "swallowed up" and ignored, instead of being propogated back to the
  // caller.
  DriverControlObj := TOTFEFreeOTFEDriverControl.Create();

end;


procedure TfrmDriverControl.EnableDisableControls();
var
  driverSelected: boolean;
  serviceRunning: boolean;
  serviceState: DWORD;
  autoStart: boolean;
begin
  // Note: We *don't* repaint/refresh lbDrivers in this procedure as that
  //       would cause a flicking effect.

  driverSelected:= (lbDrivers.ItemIndex>=0);

  SDUEnableControl(lblStart, driverSelected);
  SDUEnableControl(cbStartup, driverSelected);

  SDUEnableControl(pbStart, FALSE);
  SDUEnableControl(pbStop, FALSE);
  cbStartup.ItemIndex := -1;

  if driverSelected then
    begin
    // If the service is still running, warn the user later...
    if (DriverControlObj.GetServiceState(lbDrivers.Items[lbDrivers.ItemIndex], serviceState)) then
      begin
      serviceRunning := (serviceState = SERVICE_RUNNING);

      SDUEnableControl(pbStop, serviceRunning);
      SDUEnableControl(pbStart, not(serviceRunning));
      end;

    cbStartup.ItemIndex := cbStartup.Items.IndexOf(TEXT_MANUAL_START);
    if DriverControlObj.GetServiceAutoStart(lbDrivers.Items[lbDrivers.ItemIndex], autoStart) then
      begin
      if autoStart then
        begin
        cbStartup.ItemIndex := cbStartup.Items.IndexOf(TEXT_AUTO_START);
        end;
      end;

    end;

  SDUEnableControl(pbUninstall, driverSelected);

  SDUEnableControl(pbInstall, TRUE);

  // pbUpdate is a special case; it should only be enabled if the user
  // changes cbStartup
  SDUEnableControl(pbUpdate, FALSE);

end;


procedure TfrmDriverControl.PopulateDriversList();
var
  driverList: TStringList;
  maxWidth: integer;
  i: integer;
begin
  lbDrivers.Items.Clear();

  driverList := TStringList.Create();
  try
    DriverControlObj.GetFreeOTFEDrivers(driverList);

    // Bang the sorted flag up & down to ensure that the list is sorted
    // before it is displayed to the user
    driverList.Sorted := FALSE;
    driverList.Sorted := TRUE;

    lbDrivers.Items.AddStrings(driverList);
  finally
    driverList.Free();
  end;


  maxWidth := 0;
  for i:=0 to (lbDrivers.Items.count-1) do
    begin
    maxWidth := max(maxWidth, GetItemWidth(i));
    end;
  lbDrivers.ScrollWidth := maxWidth;


  // Ensure no driver is selected
  lbDrivers.ItemIndex := -1;
  // Ensure that the fact none is selected takes effect...
  lbDriversClick(nil);

end;


procedure TfrmDriverControl.lbDriversClick(Sender: TObject);
begin
  EnableDisableControls();

end;

procedure TfrmDriverControl.FormShow(Sender: TObject);
begin
  cbStartup.Items.Add(TEXT_MANUAL_START);
  cbStartup.Items.Add(TEXT_AUTO_START);

  PopulateDriversList();

  EnableDisableControls();

end;

procedure TfrmDriverControl.pbStartClick(Sender: TObject);
var
  service: string;
begin
  service:= lbDrivers.Items[lbDrivers.ItemIndex];
  if not(DriverControlObj.StartStopService(service, TRUE)) then
    begin
    SDUMessageDlg('Unable to start driver "'+service+'"'+CRLF+
               CRLF+
               TEXT_MAY_NEED_ADMIN,
               mtError,
               [mbOK],
               0
              );
    end;

  EnableDisableControls();

  // Ensure that the icons displayed are correct
  lbDrivers.Refresh();

end;

procedure TfrmDriverControl.pbStopClick(Sender: TObject);
var
  service: string;
begin
  // Note that we *cannot* do this sensibly, and only warn the user if one or
  // more drives are mounted; the user *could* have more than one copy of
  // FreeOTFE (or different apps using the FreeOTFE driver; e.g. different apps
  // using this component).
  // In that scenario, things could get dodgy if the user did something stupid
  // like uninstalling the main driver, while keeping the 2nd app connected to
  // it.
  if (SDUMessageDlg('WARNING!'+CRLF+
                 CRLF+
                 'Stopping a FreeOTFE driver which is currently in use (e.g. by'+CRLF+
                 'one or more mounted volumes) can lead to SYSTEM INSTABILITY.'+CRLF+
                 CRLF+
                 'This option is intended for experienced users who REALLY'+CRLF+
                 'know what they''re doing.'+CRLF+
                 CRLF+
                 'Are you sure you wish to stop the "'+lbDrivers.Items[lbDrivers.ItemIndex]+'" driver?',
                 mtWarning,
                 [mbYes, mbNo],
                 0
                ) = mrYes) then
    begin
    service:= lbDrivers.Items[lbDrivers.ItemIndex];

    if not(DriverControlObj.StartStopService(service, FALSE)) then
      begin
      SDUMessageDlg('Unable to stop driver "'+service+'"'+CRLF+
                 CRLF+
                 TEXT_MAY_NEED_ADMIN,
                 mtError,
                 [mbOK],
                 0
                );
      end;

    EnableDisableControls();
    end;

  // Ensure that the icons displayed are correct
  lbDrivers.Refresh();

end;


procedure TfrmDriverControl.pbUninstallClick(Sender: TObject);
var
  msgReboot: string;
  msgManualRemoveFile: string;
  driverName: string;
  status: DWORD;
begin
  // Note that we *cannot* do this sensibly, and only warn the user if one or
  // more drives are mounted; the user *could* have more than one copy of
  // FreeOTFE (or different apps using the FreeOTFE driver; e.g. different apps
  // using this component).
  // In that scenario, things could get dodgy if the user did something stupid
  // like uninstalling the main driver, while keeping the 2nd app connected to
  // it.
  if (SDUMessageDlg('WARNING!'+CRLF+
               CRLF+
               'Uninstalling a FreeOTFE driver which is currently in use (e.g. by'+CRLF+
               'one or more mounted volumes) can lead to SYSTEM INSTABILITY.'+CRLF+
               CRLF+
               'This option is intended for experienced users who REALLY'+CRLF+
               'know what they''re doing.'+CRLF+
               CRLF+
               'Are you sure you wish to uninstall the "'+lbDrivers.Items[lbDrivers.ItemIndex]+'" driver?',
               mtWarning,
               [mbYes, mbNo],
               0
              ) = mrYes) then
    begin
    driverName:= lbDrivers.Items[lbDrivers.ItemIndex];

    status := DriverControlObj.UninstallDriver(driverName);

    if ((status AND DRIVER_BIT_SUCCESS) = DRIVER_BIT_SUCCESS) then
      begin
      // Uninstallation SUCCESSFULL
      msgReboot := '';
      if ((status AND DRIVER_BIT_REBOOT_REQ) = DRIVER_BIT_REBOOT_REQ) then
        begin
        msgReboot :=
                     CRLF+
                     CRLF+  // 2 x CRLF - one to end the last line, one for spacing
                     'Please reboot your computer to allow changes to '+CRLF+
                     'take effect.';
        end;

      msgManualRemoveFile := '';
      if ((status AND DRIVER_BIT_FILE_REMAINS) = DRIVER_BIT_FILE_REMAINS) then
        begin
        msgManualRemoveFile :=
                               CRLF+
                               CRLF+  // 2 x CRLF - one to end the last line, one for spacing
                               'After rebooting, please remove the driver file from your '+CRLF+
                               '<windows>\system32\drivers directory';
        end;

      SDUMessageDlg(
                 'The "'+driverName+'" driver has now been uninstalled.'+
                 msgReboot+
                 msgManualRemoveFile,
                 mtInformation,
                 [mbOK],
                 0
                );
      end
    else
      begin
      // Uninstallation FAILURE
      SDUMessageDlg(
                 'Unable to delete driver service "'+driverName+'".'+CRLF+
                 CRLF+
                 'Please disable this driver from starting up automatically, reboot, '+CRLF+
                 'and try again.',
                 mtError,
                 [mbOK],
                 0
                );
      end;

    end;

  // Refresh the drivers list
  PopulateDriversList();

end;


// Install a new FreeOTFE device driver
procedure TfrmDriverControl.pbInstallClick(Sender: TObject);
var
  status: DWORD;
  msgReboot: string;
  msgAlreadyInstalled: string;
  msgCantCopyFile: string;
  driverName: string;
begin
  OpenDialog.Filter := 'Driver files (*.sys)|*.sys|All files|*.*';
  if (OpenDialog.Execute()) then
    begin
    status := DriverControlObj.InstallDriver(OpenDialog.Filename, FALSE, driverName);

    if ((status AND DRIVER_BIT_SUCCESS) = DRIVER_BIT_SUCCESS) then
      begin
      // Installation SUCCESSFULL
      msgReboot := '';
      if ((status AND DRIVER_BIT_REBOOT_REQ) = DRIVER_BIT_REBOOT_REQ) then
        begin
        msgReboot :=
                     CRLF+
                     CRLF+  // 2 x CRLF - one to end the last line, one for spacing
                     'Please reboot your computer to allow changes to '+CRLF+
                     'take effect.';
        end;

      SDUMessageDlg(
                 'Driver "'+driverName+'" installed.'+CRLF+
                 CRLF+
                 'This driver has been installed such that it will not automatically'+CRLF+
                 'start up when your computer is booted; this may be changed by'+CRLF+
                 'selecting the driver from the list below and changing it''s startup configuration.'+CRLF+
                 CRLF+
                 'You may also start the driver now by selecting it and clicking '+CRLF+
                 '"Start"'+
                 msgReboot,
                 mtInformation,
                 [mbOK],
                 0
                );
      end
    else
      begin
      // Installation FAILED
      msgReboot := '';
      if ((status AND DRIVER_BIT_REBOOT_REQ) = DRIVER_BIT_REBOOT_REQ) then
        begin
        msgReboot :=
                     CRLF+
                     CRLF+  // 2 x CRLF - one to end the last line, one for spacing
                     'Please reboot your computer to allow changes to '+CRLF+
                     'take effect.';
        end;

      msgAlreadyInstalled := '';
      if ((status AND DRIVER_BIT_ALREADY_INSTALLED) = DRIVER_BIT_ALREADY_INSTALLED) then
        begin
        msgAlreadyInstalled :=
                           CRLF+
                           CRLF+  // 2 x CRLF - one to end the last line, one for spacing
                           'This driver appears to already be installed. Please uninstall '+CRLF+
                           'the existing driver before attempting to reinstall.';
        end;

      msgCantCopyFile := '';
      if ((status AND DRIVER_BIT_CANT_INSTALL_FILE) = DRIVER_BIT_CANT_INSTALL_FILE) then
        begin
        msgCantCopyFile :=
                           CRLF+
                           CRLF+  // 2 x CRLF - one to end the last line, one for spacing
                           'Unable to copy driver file to windows driver directory';
        end;

      SDUMessageDlg(
                 'Unable to install driver.'+
                 msgAlreadyInstalled+
                 msgCantCopyFile+
                 msgReboot,
                 mtError,
                 [mbOK],
                 0
                );
      end;

    end;  // if (OpenDialog.Execute()) then

  // Refresh the drivers list
  PopulateDriversList();

end;


procedure TfrmDriverControl.pbCloseClick(Sender: TObject);
begin
  Close();

end;

procedure TfrmDriverControl.FormCreate(Sender: TObject);
begin
  Panel1.Caption := '';
  Panel2.Caption := '';

  LoadBitmapsFromResources();

end;


procedure TfrmDriverControl.cbStartupChange(Sender: TObject);
begin
  // pbUpdate is a special case; it should only be enabled if the user
  // changes cbStartup
  pbUpdate.Enabled := TRUE;

end;


procedure TfrmDriverControl.pbUpdateClick(Sender: TObject);
var
  autoStart: boolean;
  service: string;
begin
  if (cbStartup.ItemIndex>=0) then
    begin
    autoStart := (cbStartup.ItemIndex = (cbStartup.Items.IndexOf(TEXT_AUTO_START)));
    service := lbDrivers.Items[lbDrivers.ItemIndex];
    if not(DriverControlObj.SetServiceAutoStart(service, autoStart)) then
      begin
      SDUMessageDlg(
                 'Unable to set startup option for driver "'+service+'".'+CRLF+
                 CRLF+
                 TEXT_MAY_NEED_ADMIN,
                 mtError,
                 [mbOK],
                 0
                );
      end;
    end;

  EnableDisableControls();

  // Ensure that the icons displayed are correct
  lbDrivers.Refresh();

end;

procedure TfrmDriverControl.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  FreeBitmapsFromResources();

  DriverControlObj.Free();

end;



procedure TfrmDriverControl.LoadBitmapsFromResources();
var
  thehBitmap: hBitmap;
begin
  thehBitmap:= LoadBitmap(hInstance, 'MODE_NORMAL');
  BMPModeNormal:= TBitmap.Create();
  BMPModeNormal.ReleaseHandle();
  BMPModeNormal.handle := thehBitmap;

  thehBitmap:= LoadBitmap(hInstance, 'MODE_PORTABLE');
  BMPModePortable:= TBitmap.Create();
  BMPModePortable.ReleaseHandle();
  BMPModePortable.handle := thehBitmap;

  thehBitmap:= LoadBitmap(hInstance, 'STATUS_STARTED');
  BMPStatusStarted:= TBitmap.Create();
  BMPStatusStarted.ReleaseHandle();
  BMPStatusStarted.handle := thehBitmap;

  thehBitmap:= LoadBitmap(hInstance, 'STATUS_STOPPED');
  BMPStatusStopped:= TBitmap.Create();
  BMPStatusStopped.ReleaseHandle();
  BMPStatusStopped.handle := thehBitmap;

  thehBitmap:= LoadBitmap(hInstance, 'START_AUTO');
  BMPStartAuto:= TBitmap.Create();
  BMPStartAuto.ReleaseHandle();
  BMPStartAuto.handle := thehBitmap;

  thehBitmap:= LoadBitmap(hInstance, 'START_MANUAL');
  BMPStartManual:= TBitmap.Create();
  BMPStartManual.ReleaseHandle();
  BMPStartManual.handle := thehBitmap;

end;


procedure TfrmDriverControl.FreeBitmapsFromResources();
var
  thehBitmap: hBitmap;
begin
  thehBitmap := BMPModeNormal.Handle;
  BMPModeNormal.ReleaseHandle();
  BMPModeNormal.Free();
  DeleteObject(thehBitmap);

  thehBitmap := BMPModePortable.Handle;
  BMPModePortable.ReleaseHandle();
  BMPModePortable.Free();
  DeleteObject(thehBitmap);

  thehBitmap := BMPStatusStarted.Handle;
  BMPStatusStarted.ReleaseHandle();
  BMPStatusStarted.Free();
  DeleteObject(thehBitmap);

  thehBitmap := BMPStatusStopped.Handle;
  BMPStatusStopped.ReleaseHandle();
  BMPStatusStopped.Free();
  DeleteObject(thehBitmap);

  thehBitmap := BMPStartAuto.Handle;
  BMPStartAuto.ReleaseHandle();
  BMPStartAuto.Free();
  DeleteObject(thehBitmap);

  thehBitmap := BMPStartManual.Handle;
  BMPStartManual.ReleaseHandle();
  BMPStartManual.Free();
  DeleteObject(thehBitmap);

end;


// Note: The "Style" property of the TListBox must be set to lbOwnerDrawFixed
// !! IMPORTANT !!
// !! IMPORTANT !!
// lbDriversDrawItem and GetItemWidth *MUST* be kept in sync!
// !! IMPORTANT !!
// !! IMPORTANT !!
procedure TfrmDriverControl.lbDriversDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  lbObj: TListBox;
  canvas: TCanvas;
  offset: Integer;
  portableMode: boolean;
  serviceState: DWORD;
  autoStart: boolean;
  drawStatusBmp: TBitmap;
  drawModeBmp: TBitmap;
  drawStartBmp: TBitmap;
begin
  lbObj := TListBox(Control);
  canvas := lbObj.Canvas;

  offset := INTER_IMAGE_GAP;  // Default offset

  canvas.FillRect(Rect);  { clear the rectangle }

{
  // Portable Drivers in italics
  canvas.Font.Style := [];
  if DriverControlObj.IsDriverInstalledPortable(lbObj.Items[Index], portableMode) then
    begin
    if portableMode then
      begin
      canvas.Font.Style := [fsItalic];
      end;

    end;
}

  // Determine which icons are to be displayed...
  
  // Auto or manual start...
  drawStartBmp := BMPStartManual;
  if (DriverControlObj.GetServiceAutoStart(lbObj.Items[Index], autoStart)) then
    begin
    if (autoStart) then
      begin
      drawStartBmp := BMPStartAuto;
      end
    end;

  // Started or stopped...
  drawStatusBmp := BMPStatusStopped;
  if (DriverControlObj.GetServiceState(lbObj.Items[Index], serviceState)) then
    begin
    if (serviceState = SERVICE_RUNNING) then
      begin
      drawStatusBmp := BMPStatusStarted;
      end
    end;

  // Normal or portable...
  // If running normally, don't display an icon
//  drawModeBmp := BMPModeNormal;
  drawModeBmp := nil;
  if DriverControlObj.IsDriverInstalledPortable(lbObj.Items[Index], portableMode) then
    begin
    if (portableMode) then
      begin
      drawModeBmp := BMPModePortable;
      end;
    end;


  // Draw the icons...

  // Auto or manual start...
  lbObj.Canvas.Draw(
                    (Rect.Left+offset),
                    (Rect.Top + (((Rect.Bottom - Rect.Top) - drawStartBmp.Height) div 2)),
                    drawStartBmp
                   );
  offset := offset + INTER_IMAGE_GAP + max(BMPStartAuto.Width, BMPStartManual.Width);

  // Started or stopped...
  if (drawModeBmp <> nil) then
    begin
    lbObj.Canvas.Draw(
                      (Rect.Left+offset),
                      (Rect.Top + (((Rect.Bottom - Rect.Top) - drawModeBmp.Height) div 2)),
                      drawModeBmp
                     );
    end;
  offset := offset + INTER_IMAGE_GAP + max(BMPModePortable.Width, BMPModeNormal.Width);

  // Normal or portable...
  lbObj.Canvas.Draw(
                    (Rect.Left+offset),
                    (Rect.Top + (((Rect.Bottom - Rect.Top) - drawStatusBmp.Height) div 2)),
                    drawStatusBmp
                   );
  offset := offset + INTER_IMAGE_GAP + max(BMPStatusStarted.Width, BMPStatusStopped.Width);


  // The name...
  canvas.TextOut((Rect.Left+offset), Rect.Top, lbObj.Items[Index]);
           
end;


function TfrmDriverControl.GetItemWidth(Index: Integer): integer;
var
  lbObj: TListBox;
  canvas: TCanvas;
  offset: Integer;
begin
  lbObj := lbDrivers;
  canvas := lbObj.Canvas;

  offset := INTER_IMAGE_GAP;  // Default offset

  offset := offset + INTER_IMAGE_GAP + max(BMPStartAuto.Width, BMPStartManual.Width);
  offset := offset + INTER_IMAGE_GAP + max(BMPModePortable.Width, BMPModeNormal.Width);
  offset := offset + INTER_IMAGE_GAP + max(BMPStatusStarted.Width, BMPStatusStopped.Width);

  // The name...
  offset := offset + canvas.TextWidth(lbObj.Items[Index]);


  // Tack on a bit so it doesn't look like the text goes right up against the
  // end...
  offset := offset + (2 * INTER_IMAGE_GAP);

  Result := offset;

end;


END.


