unit OTFEFreeOTFE_frmKeyEntryFreeOTFE;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Spin, PasswordRichEdit, Spin64,
  OTFEFreeOTFE_U;  // Required for TFreeOTFEMountAs

type
  TfrmKeyEntryFreeOTFE = class(TForm)
    pbCancel: TButton;
    pbOK: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    cbDrive: TComboBox;
    ckMountReadonly: TCheckBox;
    GroupBox4: TGroupBox;
    Label8: TLabel;
    Label3: TLabel;
    preUserkey: TPasswordRichEdit;
    se64Offset: TSpinEdit64;
    edKeyfile: TEdit;
    Label6: TLabel;
    pbBrowse: TButton;
    OpenDialog: TOpenDialog;
    ckOffsetPointsToCDB: TCheckBox;
    Label2: TLabel;
    seSaltLength: TSpinEdit;
    Label5: TLabel;
    seKeyIterations: TSpinEdit;
    Label7: TLabel;
    cbMediaType: TComboBox;
    Label9: TLabel;
    ckMountForAllUsers: TCheckBox;
    procedure pbOKClick(Sender: TObject);
    procedure preUserkeyKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure pbCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure edKeyfileChange(Sender: TObject);
    procedure seSaltLengthChange(Sender: TObject);
    procedure seKeyIterationsChange(Sender: TObject);
    procedure cbMediaTypeChange(Sender: TObject);
  private

  protected
    procedure PopulateDrives();
    procedure PopulateMountAs();

    procedure DoCancel();

    procedure EnableDisableControls();

  public
    function  GetKey(var key: string): boolean;
    function  GetKeyfile(var keyfile: string): boolean;
    function  GetKeyIterations(var keyIterations: integer): boolean;

    function  GetDriveLetter(var driveLetter: char): boolean;
    // Offset given in bytes...
    function  GetOffset(var offset: int64): boolean;
    // Salt length given in bits...
    function  GetSaltLength(var saltLength: integer): boolean;

    procedure SetReadOnly(readonly: boolean);
    function  GetReadOnly(var readonly: boolean): boolean;
    function  GetMountAs(var mountAs: TFreeOTFEMountAs): boolean;
    function  GetMountForAllUsers(): boolean;

    function  GetOffsetPointsToCDB(var offsetPointsToCDB: boolean): boolean;
  end;


implementation

{$R *.DFM}


uses
  ComObj,  // Required for StringToGUID
  SDUGeneral,
  SDUDialogs;

const

  CRLF = #10+#13;

function TfrmKeyEntryFreeOTFE.GetKey(var key: string): boolean;
begin
  key := preUserkey.Text;
  Result := TRUE;
end;


function TfrmKeyEntryFreeOTFE.GetDriveLetter(var driveLetter: char): boolean;
begin
  driveLetter := #0;
  // Note: The item at index zero is "Use default"; #0 is returned for this
  if (cbDrive.ItemIndex>0) then
    begin
    driveLetter := cbDrive.Items[cbDrive.ItemIndex][1];
    end;

  Result := TRUE;
end;


// Offset given in bytes...
function TfrmKeyEntryFreeOTFE.GetOffset(var offset: int64): boolean;
begin
  offset := se64Offset.Value;
  Result := TRUE;
end;


// Offset given in bits...
function TfrmKeyEntryFreeOTFE.GetSaltLength(var saltLength: integer): boolean;
begin
  saltLength := seSaltLength.Value;
  Result := TRUE;
end;

function TfrmKeyEntryFreeOTFE.GetKeyIterations(var keyIterations: integer): boolean;
begin
  keyIterations := seKeyIterations.Value;
  Result := TRUE;
end;

procedure TfrmKeyEntryFreeOTFE.SetReadOnly(readonly: boolean);
begin
  ckMountReadonly.checked :=readonly;
end;


function TfrmKeyEntryFreeOTFE.GetReadOnly(var readonly: boolean): boolean;
begin
  readonly := ckMountReadonly.checked;
  Result := TRUE;
end;


function TfrmKeyEntryFreeOTFE.GetMountAs(var mountAs: TFreeOTFEMountAs): boolean;
var
  currMountAs: TFreeOTFEMountAs;
  allOK: boolean;
begin
  allOK := FALSE;

  for currMountAs:=low(TFreeOTFEMountAs) to high(TFreeOTFEMountAs) do
    begin
    if (cbMediaType.Items[cbMediaType.ItemIndex] = FreeOTFEMountAsTitle[currMountAs]) then
      begin
      mountAs := currMountAs;
      allOK := TRUE;
      break;
      end;
    end;

  Result := allOK;
end;

function TfrmKeyEntryFreeOTFE.GetMountForAllUsers(): boolean;
begin
  Result := ckMountForAllUsers.checked;
end;


procedure TfrmKeyEntryFreeOTFE.PopulateDrives();
var
  driveLetters: string;
  i: integer;
begin
  cbDrive.Items.Clear();
  cbDrive.Items.Add('Use default');
  driveLetters := SDUGetUnusedDriveLetters();
  for i:=1 to length(driveLetters) do
    begin
    // Skip the drive letters traditionally reserved for floppy disk drives
    if (
        (driveLetters[i] <> 'A') AND
        (driveLetters[i] <> 'B')
       ) then
      begin
      cbDrive.Items.Add(driveLetters[i]+':');
      end;
    end;

end;


procedure TfrmKeyEntryFreeOTFE.PopulateMountAs();
var
  currMountAs: TFreeOTFEMountAs;
begin
  cbMediaType.Items.Clear();
  for currMountAs:=low(TFreeOTFEMountAs) to high(TFreeOTFEMountAs) do
    begin
    if (currMountAs <> fomaUnknown) then
      begin
      cbMediaType.Items.Add(FreeOTFEMountAsTitle[currMountAs]);
      end;

    end;

end;


procedure TfrmKeyEntryFreeOTFE.pbOKClick(Sender: TObject);
begin

  if (seSaltLength.value mod 8 <> 0) then
    begin
    if (SDUMessageDlg(
               'Salt length (in bits) must be a multiple of 8)',
               mtError,
               [mbOK],
               0
              ) = mrYes) then
      begin
      ModalResult := mrOK;
      end;
    end
  else
    begin
    ModalResult := mrOK;
    end;

end;



procedure TfrmKeyEntryFreeOTFE.preUserkeyKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (key = 27) then
    begin
    DoCancel();
    end;

end;

procedure TfrmKeyEntryFreeOTFE.pbCancelClick(Sender: TObject);
begin
  DoCancel();

end;


procedure TfrmKeyEntryFreeOTFE.DoCancel();
begin
  ModalResult := mrCancel;

end;


procedure TfrmKeyEntryFreeOTFE.FormShow(Sender: TObject);
begin
  preUserKey.Plaintext := TRUE;
  // FreeOTFE volumes CAN have newlines in the user's password
  preUserKey.WantReturns := TRUE;
  preUserKey.WordWrap := TRUE;
  preUserKey.Lines.Clear();
  preUserKey.PasswordChar := '*';

  seSaltLength.Increment := 8;
  seSaltLength.Value := DEFAULT_SALT_LENGTH;

  seKeyIterations.MinValue := 1;
  seKeyIterations.MaxValue := 999999; // Need *some* upper value, otherwise setting MinValue won't work properly
  seKeyIterations.Increment := DEFAULT_KEY_ITERATIONS_INCREMENT;
  seKeyIterations.Value := DEFAULT_KEY_ITERATIONS;

  edKeyfile.Text := '';
  ckOffsetPointsToCDB.Checked := TRUE;

  PopulateDrives();
  if (cbDrive.Items.Count>0) then
    begin
    cbDrive.ItemIndex := 0;
    end;

  PopulateMountAs();
  if (cbMediaType.Items.Count>0) then
    begin
    cbMediaType.ItemIndex := 0;
    end;

  // Default to TRUE to allow formatting under Windows Vista
  ckMountForAllUsers.checked := TRUE;

  EnableDisableControls();

end;

procedure TfrmKeyEntryFreeOTFE.EnableDisableControls();
var
  ki: integer;
  sl: integer;
  mountAsOK: boolean;
  tmpMountAs: TFreeOTFEMountAs;
begin
  // If no keyfile is specified, then the CDB must reside within the volume
  // file
  if (edKeyfile.Text = '') then
    begin
    ckOffsetPointsToCDB.Enabled := FALSE;
    ckOffsetPointsToCDB.Checked := TRUE;
    end
  else
    begin
    // If a keyfile is specified, then the user can specify if the volume file
    // includes a CDB
    ckOffsetPointsToCDB.Enabled := TRUE;
    end;

  GetKeyIterations(ki);
  GetSaltLength(sl);


  // Ensure we know what to mount as
  mountAsOK := GetMountAs(tmpMountAs);
  ckMountReadonly.Enabled := FALSE;
  if (mountAsOK) then
    begin
    if not(FreeOTFEMountAsCanWrite[tmpMountAs]) then
      begin
      ckMountReadonly.checked := TRUE;
      end;
    SDUEnableControl(ckMountReadonly, FreeOTFEMountAsCanWrite[tmpMountAs]);
    end;
  mountAsOK := mountAsOK AND (tmpMountAs <> fomaUnknown);


  pbOK.Enabled := mountAsOK AND
                  (cbDrive.ItemIndex >= 0) AND
                  (ki > 0) AND
                  (sl >= 0);
end;



function TfrmKeyEntryFreeOTFE.GetKeyfile(var keyfile: string): boolean;
begin
  keyfile := edKeyfile.Text;

  Result := TRUE;
end;


function TfrmKeyEntryFreeOTFE.GetOffsetPointsToCDB(var offsetPointsToCDB: boolean): boolean;
begin
  offsetPointsToCDB := ckOffsetPointsToCDB.checked;

  Result := TRUE;
end;

procedure TfrmKeyEntryFreeOTFE.pbBrowseClick(Sender: TObject);
begin
  OpenDialog.Filter     := FILE_FILTER_FLT_KEYFILES;
  OpenDialog.DefaultExt := FILE_FILTER_DFLT_NONE;

  SDUOpenSaveDialogSetup(OpenDialog, edKeyfile.Text);
  if OpenDialog.Execute then
    begin
    edKeyfile.Text := OpenDialog.Filename;
    EnableDisableControls();
    end;

end;

procedure TfrmKeyEntryFreeOTFE.edKeyfileChange(Sender: TObject);
begin
  EnableDisableControls();

end;

procedure TfrmKeyEntryFreeOTFE.seSaltLengthChange(Sender: TObject);
begin
  EnableDisableControls();
end;

procedure TfrmKeyEntryFreeOTFE.seKeyIterationsChange(Sender: TObject);
begin
  EnableDisableControls();
end;


procedure TfrmKeyEntryFreeOTFE.cbMediaTypeChange(Sender: TObject);
begin
  EnableDisableControls();
  
end;

END.


