unit SDUURLLabel;
// Description: URL Label
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//
// Set "URL" to be the URL to be launched on clicking; NormalColor and
// HoverColor to the font color to be used (these colors override any set
// directly on the font)
//
// -----------------------------------------------------------------------------


interface

uses
  stdctrls, Classes, Graphics, Messages, Controls,
  Windows, ShellAPI;

type

  TSDUURLLabel = class(TLabel)

  private
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    FHoverColor: TColor;
    FNormalColor: TColor;
    FURL: string;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;

    procedure Click(); override;

  published
    property NormalColor: TColor read FNormalColor write FNormalColor;
    property HoverColor: TColor read FHoverColor write FHoverColor;
    property URL: string read FURL write FURL;

  end;

procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('SDeanUtils', [TSDUURLLabel]);
end;

constructor TSDUURLLabel.Create(AOwner: TComponent);
begin
  inherited;
  NormalColor:= clBlue;
  HoverColor:= clRed;
  URL := '';

  Font.Color := NormalColor;
  Font.Style := Font.Style + [fsUnderline];

  Cursor := crHandPoint;

end;


destructor TSDUURLLabel.Destroy();
begin

  inherited;
end;


procedure TSDUURLLabel.CMMouseEnter(var Message: TMessage);
begin
  inherited;

  Font.Color := HoverColor;
  Repaint;

end;

procedure TSDUURLLabel.CMMouseLeave(var Message: TMessage);
begin
  inherited;

  Font.Color := NormalColor;
  Invalidate;

end;


procedure TSDUURLLabel.Click();
begin
  inherited;

  if (URL <> '') then
    begin
    ShellExecute(
                 Parent.Handle,
                 PChar('open'),
                 PChar(URL),
                 PChar(''),
                 PChar(''),
                 SW_SHOW
                );
    end;
    
end;

END.


