unit RNGTest_U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, MouseRNG;

type
  TRNGTest = class(TForm)
    pbClose: TButton;
    reReport: TRichEdit;
    rgRNG: TRadioGroup;
    pbGenerateRandomData: TButton;
    pbClear: TButton;
    edGPGFilename: TEdit;
    lblGPGFilename: TLabel;
    MouseRNG: TMouseRNG;
    lblMouseRNGBits: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    procedure pbCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure pbGenerateRandomDataClick(Sender: TObject);
    procedure pbClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure rgRNGClick(Sender: TObject);
    procedure MouseRNGByteGenerated(Sender: TObject; random: Byte);
  private
    CanUseCryptlib: boolean;

    fRandomData: string;
  public
    { Public declarations }
  end;

var
  RNGTest: TRNGTest;

implementation

{$R *.DFM}

uses
  OTFEFreeOTFE_WizardCommon,
  OTFEFreeOTFE_DriverAPI,  // Required for CRITICAL_DATA_LENGTH
  SDUGeneral;

procedure TRNGTest.pbCloseClick(Sender: TObject);
begin
  Close();

end;

procedure TRNGTest.FormShow(Sender: TObject);
begin
  rgRNG.Items.Clear();
  rgRNG.Items.Add(RNG_OPT_MS_CRYPTOAPI);
  rgRNG.Items.Add(RNG_OPT_MOUSE_MOVEMENT);
  rgRNG.Items.Add(RNG_OPT_CRYPTLIB);
  rgRNG.Items.Add(RNG_OPT_GPG);
  rgRNG.ItemIndex := rgRNG.Items.IndexOf(DEFAULT_RNG);
  rgRNG.Controls[(rgRNG.Items.IndexOf(RNG_OPT_CRYPTLIB))].Enabled := CanUseCryptlib;

  reReport.Lines.Clear();

  Panel1.Caption := '';
  Panel2.Caption := '';
  Panel3.Caption := '';
end;

procedure TRNGTest.pbGenerateRandomDataClick(Sender: TObject);
var
  allOK: boolean;
  tmpStringList: TStringList;
begin
  allOK := GenerateRandomData(
                              rgRNG.Items[rgRNG.ItemIndex],
                              (CRITICAL_DATA_LENGTH div 8),
                              edGPGFilename.Text,
                              fRandomData
                             );
  if (allOK) then
    begin
    reReport.Lines.Add('SUCCESS: Random data generated');

    tmpStringList := TStringList.Create();
    try
      SDUPrettyPrintHex(
                        fRandomData,
                        0,
                        length(fRandomData),
                        tmpStringList
                       );

      reReport.Lines.AddStrings(tmpStringList);
    finally
      tmpStringList.Free();
    end;

    end
  else
    begin
    reReport.Lines.Add('FAILED: Random data NOT generated');
    end;

end;

procedure TRNGTest.pbClearClick(Sender: TObject);
begin
  reReport.Lines.Clear();

end;

procedure TRNGTest.FormCreate(Sender: TObject);
begin
  fRandomData:= '';
  
  // Start cryptlib, if possible, as early as we can to allow it as much time
  // as possible to poll entropy
  CanUseCryptlib := cryptlibLoad();

end;

procedure TRNGTest.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  // Shutdown cryptlib, if used
  if CanUseCryptlib then
    begin
    cryptlibUnload();
    CanUseCryptlib := FALSE;
    end;

end;


procedure TRNGTest.rgRNGClick(Sender: TObject);
begin
  fRandomData := '';


  // Just for testing...
  SDUEnableControl(MouseRNG, (rgRNG.Items[rgRNG.ItemIndex] = RNG_OPT_MOUSE_MOVEMENT));
  SDUEnableControl(lblMouseRNGBits, (rgRNG.Items[rgRNG.ItemIndex] = RNG_OPT_MOUSE_MOVEMENT));
  
  SDUEnableControl(lblGPGFilename, (rgRNG.Items[rgRNG.ItemIndex] = RNG_OPT_GPG));
  SDUEnableControl(edGPGFilename, (rgRNG.Items[rgRNG.ItemIndex] = RNG_OPT_GPG));

end;

procedure TRNGTest.MouseRNGByteGenerated(Sender: TObject; random: Byte);
var
  randomBitsGenerated: integer;
  allOK: boolean;
begin
  // Note: This is correct; if it's *less than* CRITICAL_DATA_LEN, then store it
  if ((length(fRandomData) * 8) < CRITICAL_DATA_LENGTH) then
    begin
    fRandomData := fRandomData + char(random);
    end;


  // This is a good place to update the display of the number of random bits
  // generated...
  randomBitsGenerated:= (length(fRandomData) * 8);
  lblMouseRNGBits.Caption := 'Random bits generated: '+inttostr(randomBitsGenerated)+'/'+inttostr(CRITICAL_DATA_LENGTH);

  allOK := (randomBitsGenerated >= CRITICAL_DATA_LENGTH);
  MouseRNG.Enabled := not(allOK);
  if MouseRNG.Enabled then
    begin
    MouseRNG.Color := clWindow;
    end
  else
    begin
    MouseRNG.Color := clBtnFace;
    end;


end;

END.


